// TYPE: C
// PRJ: YAO
// PGM: YAO
// VER: 1.3.1
// USES: Visual C++ 7.0, Xcode 2.3
// CREATE: 20040126 (iwai)
// UPDATE: 20041216 (yamamoto, sugawara)
// UPDATE: 20050831 (shiina, yatou)
// UPDATE: 20060228 (yamamoto)
// UPDATE: 20060907 (asami)
//////////////////////////////////////////////////
// 20041216 version 1.1.0 STL廃止。Cのラッパー廃止。全面的にCで書き直し。
// 20050624 version 1.2.1 Windows依存部分を標準ライブラリ関数を用いて書き直し。
//                        現在のメタデータ(通称「カレント」)をYAO_FILEに保持しながら
//                        操作するように仕様変更。アクセスモードを指定できるようになった。
// 20060228 version 1.3.0 YAO_ReplaceData 追加
// 20060907 version 1.3.1 YAOStatusをYAO_STATUSに,YAOTagをYAO_TAGに変更

//#include "stdafx.h"
#include "YAO.h"

//**************************************
// プロトタイプ宣言
//**************************************
// ファイル操作関係の API をラッピング
static bool MyIsFileExists( const char * inFilePath );
static YAO_STATUS MySetFilePointer( YAO_FILE * ioFile, long inPosition );
static YAO_STATUS MySetFilePointerToEnd( YAO_FILE * ioFile );
static long MyGetCurrentFilePointer( YAO_FILE * ioFile );
static YAO_STATUS MyReadData( YAO_FILE * ioFile, void * outBuff, long inBufLen );
static YAO_STATUS MyWriteData( YAO_FILE * ioFile, const void * inBuff, long inBufLen );
static bool MyIsValidYAO_FILE( YAO_FILE * ioFile );

// YAO_METADATA 関連の関数
static void InitMetaData( YAO_METADATA * ioMetaDataPtr );
static bool IsTagObj( YAO_METADATA inMetaData );
static YAO_STATUS ReadMetaData( YAO_FILE * ioFile, long inPosition, YAO_METADATA * outMetaDataPtr );
static YAO_STATUS WriteMetaData( YAO_FILE * ioFile, YAO_METADATA * inMetaDataPtr );

static YAO_STATUS FindFirstTagObj( YAO_FILE * ioFile, YAO_METADATA * outMetaDataPtr ); 
static YAO_STATUS FindNextTagObj( YAO_FILE * ioFile, YAO_METADATA * ioMetaDataPtr ); 
static YAO_STATUS FindPreviousTagObj( YAO_FILE * ioFile, YAO_METADATA * ioMetaDataPtr ); 
static YAO_STATUS FindLastTagObj( YAO_FILE * ioFile, YAO_METADATA * outMetaDataPtr );
static YAO_STATUS FindNextMember( YAO_FILE * ioFile, YAO_METADATA * ioMetaDataPtr );
static YAO_STATUS FindPreviousMember( YAO_FILE * ioFile, YAO_METADATA * ioMetaDataPtr );
static YAO_STATUS FindFirstMember( YAO_FILE * ioFile, YAO_TAG inTag, YAO_METADATA * outMetaDataPtr );
static YAO_STATUS FindLastMember( YAO_FILE * ioFile, YAO_TAG inTag, YAO_METADATA * outMetaDataPtr );

static YAO_STATUS AddTagObj( YAO_FILE * ioFile, YAO_TAG inTag, long inDataLen, void * inDataBuff );
static YAO_STATUS AddMemberObj( YAO_FILE * ioFile, YAO_TAG inTag, long inDataLen, void * inDataBuff );

static YAO_STATUS RemoveTagObj( YAO_FILE * ioFile );
static YAO_STATUS RemoveMemberObj( YAO_FILE * ioFile );

static YAO_STATUS InsertMemberObj( YAO_FILE * ioFile, long inDataLen, void * inDataBuff );
static YAO_STATUS InsertTagObj( YAO_FILE * ioFile, long inDataLen, void * inDataBuff );

static YAO_STATUS UpdateNextTag( YAO_FILE * ioFile, long inPreviousTagPos );
static YAO_STATUS UpdatePreviousTag( YAO_FILE * ioFile, long inNextTagPos );
static YAO_STATUS UpdateNextMember( YAO_FILE * ioFile, long inPreviousTagPos, long inNextTagPos );

//**************************************
// 関数
//**************************************
//------------------------------------------------
// 関数 : YAO_Create
// 機能 : YAO ファイルを新しく作成して開く
// 返値 : kYAO_Success			... 成功
//        kYAO_Failure			... 失敗
//        kYAO_FileIO_Err		... ファイル入出力エラー
//------------------------------------------------
// 20050912 mod (shiina, yatou) fopenを使用するよう変更
//                              YAO_FILEを返り値から引数に変更
//                              返り値をYAO_STATUSに変更
YAO_STATUS YAO_Create( const char * inFilePath, YAO_FILE * outFile )
{
	YAO_STATUS 		rc; // return code
	YAO_FILE_HEADER	fileHeader;
	YAO_METADATA	entryPointMetaData;

	// ファイル名が指定されているか？
	if( inFilePath == NULL )
	{
		return kYAO_Failure;
	}

	// YAO_FILEが指定されているか？
	if( outFile == NULL )
	{
		return kYAO_Failure;
	}

	// YAO_FILEを初期化
	outFile->accessMode = kYAO_ReadWrite;
	outFile->filePointer = NULL;
	outFile->currentExist = false; // はじめはカレントなし

	// ファイルが既に存在する場合は失敗
	if ( MyIsFileExists( inFilePath ) == true )
	{
		return kYAO_Failure;
	}

	// ファイルを読み書き両用、バイナリモードで開く
	outFile->filePointer = fopen( inFilePath, "w+b" );
	if ( outFile->filePointer == NULL )
	{
		return kYAO_Failure;
	}

	// ファイルヘッダを作成
	memset( &fileHeader, 0, sizeof(YAO_FILE_HEADER) );
	fileHeader.signature[0] = 'Y';
	fileHeader.signature[1] = 'A';
	fileHeader.signature[2] = 'O';
	fileHeader.version = kYAOVersion;
	
	// ファイルヘッダを書き込む
	rc = MyWriteData( outFile, &fileHeader, sizeof(YAO_FILE_HEADER) );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	// エントリーポイント用のメタデータを生成
	InitMetaData( &entryPointMetaData );
	entryPointMetaData.dataTag = kEntryPointTag;
	entryPointMetaData.filePointer = MyGetCurrentFilePointer( outFile );

	// エントリーポイントをファイルに書き込む
	rc = MyWriteData( outFile, &entryPointMetaData, sizeof(YAO_METADATA) );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	return kYAO_Success;
}

//------------------------------------------------
// 関数 : YAO_Open
// 機能 : YAO ファイルを開く
// 返値 : kYAO_Success			... 成功
//        kYAO_Failure			... 失敗
//        kYAO_FileIO_Err		... ファイル入出力エラー
//------------------------------------------------
// 20050912 mod (shiina, yatou) fopenを使用するよう変更
//                              YAO_FILEを返り値から引数に変更
//                              アクセスモードを指定できるように変更
//                              返り値をYAO_STATUSに変更
YAO_STATUS YAO_Open( const char * inFilePath, long inAccessMode, YAO_FILE * outFile )
{
	YAO_FILE_HEADER	fileHeader;
	YAO_STATUS 		rc; // return code

	// ファイル名が指定されているか？
	if( inFilePath == NULL )
	{
		return kYAO_Failure;
	}

	// YAO_FILEが指定されているか？
	if( outFile == NULL )
	{
		return kYAO_Failure;
	}

	// YAO_FILEを初期化
	outFile->accessMode = inAccessMode;
	outFile->filePointer = NULL;
	outFile->currentExist = false; //はじめはカレントなし

	// アクセスモードにしたがってファイルを開く
	if( inAccessMode == kYAO_ReadWrite )
	{
		// ファイルを読み書き両用、バイナリモードで開く
		outFile->filePointer = fopen( inFilePath, "r+b" );
		if( outFile->filePointer == NULL )
		{
			return kYAO_Failure;
		}
	}
	else if( inAccessMode == kYAO_ReadOnly )
	{
		// ファイルを読み取り専用、バイナリモードで開く
		outFile->filePointer = fopen( inFilePath, "rb" );
		if( outFile->filePointer == NULL )
		{
			return kYAO_Failure;
		}
	}
	else
	{
		return kYAO_Failure;
	}

	// ファイルヘッダを読み出して YAO ファイルかどうか確認する
	rc = MyReadData( outFile, &fileHeader, sizeof(YAO_FILE_HEADER) );
	if( rc != kYAO_Success )
	{
		return rc;
	}
	if( fileHeader.signature[0] != 'Y' )
	{
		return kYAO_Failure;	// 正しい YAO ファイルでない
	}
	if( fileHeader.signature[1] != 'A' )
	{
		return kYAO_Failure;	// 正しい YAO ファイルでない
	}
	if( fileHeader.signature[2] != 'O' )
	{
		return kYAO_Failure;	// 正しい YAO ファイルでない
	}
	if( fileHeader.version != kYAOVersion )
	{
		return kYAO_Failure;	// 異なるバージョンである
	}

	return kYAO_Success;
}


//-------------------------------------------------
// 名前 : YAO_Close
// 機能 : YAO ファイルを閉じる
//-------------------------------------------------
// 20050912 mod (shiina, yatou) fcloseを使用するよう変更
//                              YAO_FILEの領域開放を行わないようにした
void YAO_Close( YAO_FILE * ioFile )
{	
	// YAOファイルが指定されているか？
	if( ioFile == NULL )
	{
		return;
	}
	
	// ファイルを閉じる
	if( ioFile->filePointer != NULL )
	{
		fclose( ioFile->filePointer );
	}
}

//------------------------------------------------
// 関数 : YAO_AddData
// 機能 : 指定タグにデータを追加する
// 返値 : kYAO_Success			... 成功
//        kYAO_Failure			... 失敗
//        kYAO_FileIO_Err		... ファイル入出力エラー
//        kYAO_FileIsReadOnly	... 読み取り専用モード
//------------------------------------------------
// 20050912 mod (shiina, yatou) 内部処理をAddMemberObjとAddTagObjに分割
//                              引数inDataBuffを最後に渡すように変更
//                              読み取り専用モードならば直ちに終了するように変更
YAO_STATUS YAO_AddData( YAO_FILE * ioFile, YAO_TAG inTag, long inDataLen, void * inDataBuff )
{
	YAO_STATUS rc; // return code

	// 有効なYAOファイル？
	if( MyIsValidYAO_FILE( ioFile ) == false )
	{
		return kYAO_Failure;
	}
	
	// 読み取り専用モードであるか？
	if( ioFile->accessMode == kYAO_ReadOnly )
	{
		return kYAO_FileIsReadOnly;
	}

	// 入力タグがエントリーポイントのタグなら失敗
	if( inTag == kEntryPointTag ) 
	{
		return kYAO_Failure;
	}
	
	// 入力バッファが指定されているか？
	if( inDataBuff == NULL )
	{
		return kYAO_Failure;
	}

	// 指定したタグオブジェクトのメンバに追加
	rc = AddMemberObj( ioFile, inTag, inDataLen, inDataBuff );
	if( rc == kYAO_TagNotFound )  // 指定タグのメンバが一つも無い
	{
		rc = AddTagObj( ioFile, inTag, inDataLen, inDataBuff ); // 新しいタグオブジェクトを追加
	}
	
	return rc;
}

//------------------------------------------------
// 関数 : YAO_InsertData
// 機能 : データをカレントの前に挿入する
// 返値 : kYAO_Success			... 成功
//        kYAO_Failure			... 失敗
//        kYAO_FileIO_Err		... ファイル入出力エラー
//        kYAO_NoCurrent		... カレントが無い
//        kYAO_FileIsReadOnly	... 読み取り専用モード
//------------------------------------------------
// 20050912 mod (shiina, yatou) カレントの前の位置にデータを挿入するように変更
//                              引数inDataBuffを最後に渡すように変更
//                              読み取り専用モードならば直ちに終了するように変更
YAO_STATUS YAO_InsertData( YAO_FILE * ioFile, long inDataLen, void * inDataBuff )
{
	YAO_STATUS	rc; // return code

	// 有効なYAOファイル？
	if( MyIsValidYAO_FILE( ioFile ) == false )
	{
		return kYAO_Failure;
	}
	
	// 読み取り専用モードであるか？
	if( ioFile->accessMode == kYAO_ReadOnly )
	{
		return kYAO_FileIsReadOnly;
	}

	// 入力バッファが指定されているか？
	if( inDataBuff == NULL )
	{
		return kYAO_Failure;
	}
	
	// カレントがあるか？
	if( ioFile->currentExist == false )
	{
		return kYAO_NoCurrent;
	}

	// カレントがタグなら、メンバの先頭にデータを挿入
	// カレントがメンバなら、カレントメンバの直前にデータを挿入
	if( IsTagObj( ioFile->currentMetaData ) == true )
	{
		rc = InsertTagObj( ioFile, inDataLen, inDataBuff );
	}
	else
	{
		rc = InsertMemberObj( ioFile, inDataLen, inDataBuff );
	}

	return rc;
}

//------------------------------------------------
// 関数 : YAO_RemoveData
// 機能 : カレントの位置にあるデータを削除する
// 返値 : kYAO_Success			... 成功
//        kYAO_Failure			... 失敗
//        kYAO_FileIO_Err		... ファイル入出力エラー
//        kYAO_NoCurrent		... カレントが無い
//        kYAO_FileIsReadOnly	... 読み取り専用モード
//------------------------------------------------
// 20050912 mod (shiina, yatou) カレントのデータを削除するように変更
//                              読み取り専用モードならば直ちに終了するように変更
YAO_STATUS YAO_RemoveData( YAO_FILE * ioFile )
{
	YAO_STATUS	rc; // return code
	
	// 有効なYAOファイル？
	if( MyIsValidYAO_FILE( ioFile ) == false )
	{
		return kYAO_Failure;
	}
	
	// 読み取り専用モードであるか？
	if( ioFile->accessMode == kYAO_ReadOnly )
	{
		return kYAO_FileIsReadOnly;
	}

	// カレントがあるか？
	if( ioFile->currentExist == false )
	{
		return kYAO_NoCurrent;
	}

	// カレントがタグならメンバの先頭を削除
	// カレントがタグでないなら、カレントのメンバを削除
	if( IsTagObj( ioFile->currentMetaData ) == true )
	{
		rc = RemoveTagObj( ioFile );
	}
	else
	{
		rc = RemoveMemberObj( ioFile );
	}

	return rc;
}

//------------------------------------------------
// 関数 : YAO_ReplaceData
// 機能 : カレントの位置にあるデータを置き換える
// 返値 : kYAO_Success			... 成功
//        kYAO_Failure			... 失敗
//        kYAO_FileIO_Err		... ファイル入出力エラー
//        kYAO_NoCurrent		... カレントが無い
//        kYAO_FileIsReadOnly	... 読み取り専用モード
//------------------------------------------------
// 20060228 add (yamamoto) 実装
YAO_STATUS YAO_ReplaceData( YAO_FILE * ioFile, long inDataLen, void * inDataBuff )
{
	YAO_STATUS rc; // return code

	// 有効なYAOファイル？
	if( MyIsValidYAO_FILE( ioFile ) == false )
	{
		return kYAO_Failure;
	}

	// 読み取り専用モードであるか？
	if( ioFile->accessMode == kYAO_ReadOnly )
	{
		return kYAO_FileIsReadOnly;
	}

	// カレントがあるか？
	if( ioFile->currentExist == false )
	{
		return kYAO_NoCurrent;
	}

	// 与えられたデータのサイズはカレントのサイズと同じか？
	if( inDataLen != ioFile->currentMetaData.length )
	{
		return kYAO_Failure; // サイズが違うので置換できない
	}

	//---- データを置き換える ----
	
	// 実データの位置を取得
	rc = MySetFilePointer( ioFile, ioFile->currentMetaData.filePointer + sizeof(YAO_METADATA) );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	// 実データを置き換え
	rc = MyWriteData( ioFile, inDataBuff, inDataLen );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	return kYAO_Success;
}

//------------------------------------------------
// 関数 : YAO_GetData
// 機能 : カレントのデータを取得する
// 返値 : kYAO_Success		... 成功
//        kYAO_Failure		... 失敗
//        kYAO_FileIO_Err	... ファイル入出力エラー
//        kYAO_NoCurrent	... カレントが無い
//------------------------------------------------
// 20050831 mod (shiina, yatou) カレントのデータを取得するように変更
YAO_STATUS YAO_GetData( YAO_FILE * ioFile, long inBufferLen, void * outBuff )
{
	YAO_STATUS	rc; // return code

	// 有効なYAOファイル？
	if( MyIsValidYAO_FILE( ioFile ) == false )
	{
		return kYAO_Failure;
	}
	
	// 出力先バッファが指定されているか？
	if( outBuff == NULL )
	{
		return kYAO_Failure;
	}
	
	// カレントがあるか？
	if( ioFile->currentExist == false )
	{
		return kYAO_NoCurrent;
	}

	// 用意されたバッファのサイズは十分か？
	if( ioFile->currentMetaData.length > inBufferLen )
	{
		return kYAO_Failure;
	}

	// 与えられたメタデータの実データに移動
	rc = MySetFilePointer( ioFile, ioFile->currentMetaData.filePointer + sizeof(YAO_METADATA) );
	if( rc != kYAO_Success )
	{
		return rc;
	}
	
	// 実データを取得
	rc = MyReadData( ioFile, outBuff, ioFile->currentMetaData.length );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	return kYAO_Success;
}


//------------------------------------------------
// 関数 : YAO_GetDataLength
// 機能 : カレントのデータの長さを取得
// 返値 : カレントのデータの長さが返る
//        エラー時やカレントが無いと -1 が返る
//------------------------------------------------
// 20050831 mod (shiina, yatou) カレントのデータの長さを取得するように変更
long YAO_GetDataLength( YAO_FILE * ioFile )
{
	// 有効なYAOファイル？
	if( MyIsValidYAO_FILE( ioFile ) == false )
	{
		return -1;
	}
	
	// カレントがあるか？
	if( ioFile->currentExist == false )
	{
		return -1;
	}
	
	return ioFile->currentMetaData.length;
}

//------------------------------------------------
// 関数 : YAO_FirstMember
// 機能 : 指定タグの最初のメンバに移動
// 返値 : kYAO_Success		... 成功
//        kYAO_Failure		... 失敗
//        kYAO_FileIO_Err	... ファイル入出力エラー
//        kYAO_TagNotFound	... 指定タグが見つからなかった
//------------------------------------------------
// 20050831 mod (shiina, yatou) 内部関数FindFirstMemberを使用するように変更
YAO_STATUS YAO_FirstMember( YAO_FILE * ioFile, YAO_TAG inTag )
{
	YAO_STATUS 		rc; // return code;
	YAO_METADATA 	metadata;

	// 有効なYAOファイル？
	if( MyIsValidYAO_FILE( ioFile ) == false )
	{
		return kYAO_Failure;
	}

	// 指定タグの最初のメンバを取得し、それをカレントに設定
	rc = FindFirstMember( ioFile, inTag, &metadata );
	if( rc == kYAO_Success )
	{
		ioFile->currentMetaData = metadata;
		ioFile->currentExist = true;

	}
	
	return rc;
}

//------------------------------------------------
// 関数 : YAO_NextMember
// 機能 : 次のメンバに移動
// 返値 : kYAO_Success			... 成功
//        kYAO_Failure			... 失敗
//        kYAO_FileIO_Err		... ファイル入出力エラー
//        kYAO_MemberNotFound	... 次のメンバが見つからなかった
//        kYAO_NoCurrent		... カレントが無い
//------------------------------------------------
// 20050831 mod (shiina, yatou) カレントをその次のメンバに移動するように変更
//                              内部関数FindNextMemberを使用するように変更
YAO_STATUS YAO_NextMember( YAO_FILE * ioFile )
{
	YAO_STATUS 		rc; // return code
	YAO_METADATA 	metadata;

	// 有効なYAOファイル？
	if( MyIsValidYAO_FILE( ioFile ) == false ) 
	{
		return kYAO_Failure;
	}
	
	// カレントがあるか？
	if( ioFile->currentExist == false )
	{
		return kYAO_NoCurrent;
	}

	// カレントの次のメンバを取得し、それをカレントに設定
	metadata = ioFile->currentMetaData;
	rc = FindNextMember( ioFile, &metadata );
	if( rc == kYAO_Success )
	{
		ioFile->currentMetaData = metadata;
	}
	
	return rc;
}

//------------------------------------------------
// 関数 : YAO_PreviousMember
// 機能 : 前のメンバに移動
// 返値 : kYAO_Success			... 成功
//        kYAO_Failure			... 失敗
//        kYAO_FileIO_Err		... ファイル入出力エラー
//        kYAO_MemberNotFound	... 前のメンバが見つからなかった
//        kYAO_NoCurrent		... カレントが無い
//------------------------------------------------
// 20050831 mod (shiina, yatou) カレントをその前のメンバに移動するように変更
//                              内部関数FindPreviousMemberを使用するように変更
YAO_STATUS YAO_PreviousMember( YAO_FILE * ioFile )
{
	YAO_STATUS 		rc; // return code
	YAO_METADATA	metadata;

	// 有効なYAOファイル？
	if( MyIsValidYAO_FILE( ioFile ) == false ) 
	{
		return kYAO_Failure;
	}
	
	// カレントがあるか？
	if( ioFile->currentExist == false )
	{
		return kYAO_NoCurrent;
	}

	// カレントの前のデータを取得し、それをカレントに設定
	metadata = ioFile->currentMetaData;
	rc = FindPreviousMember( ioFile, &metadata );
	if( rc == kYAO_Success )
	{
		ioFile->currentMetaData = metadata;
	}

	return rc;
}

//------------------------------------------------
// 関数 : YAO_FirstTag
// 機能 : 最初のタグオブジェクトに移動
// 返値 : kYAO_Success		... 成功
//        kYAO_Failure		... 失敗
//        kYAO_FileIO_Err	... ファイル入出力エラー
//        kYAO_TagNotFound	... 1つもタグが無い
//------------------------------------------------
// 20050831 add (shiina, yatou)
YAO_STATUS YAO_FirstTag( YAO_FILE * ioFile )
{
	YAO_STATUS 		rc; // return code
	YAO_METADATA	metaData;

	// 有効なYAOファイル？
	if( MyIsValidYAO_FILE( ioFile ) == false )
	{
		return kYAO_Failure;
	}

	// エントリーポイントを取得
	rc = FindFirstTagObj( ioFile, &metaData );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	// エントリーポイントの次のタグオブジェクトを取得
	rc = FindNextTagObj( ioFile, &metaData );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	// 取得したタグオブジェクトをカレントに設定
	ioFile->currentExist = true;
	ioFile->currentMetaData = metaData;

	return kYAO_Success;
}

//------------------------------------------------
// 関数 : YAO_NextTag
// 機能 : 次のタグオブジェクトに移動
// 返値 : kYAO_Success		... 成功
//        kYAO_Failure		... 失敗
//        kYAO_FileIO_Err	... ファイル入出力エラー
//        kYAO_NoCurrent	... カレントが無い
//        kYAO_TagNotFound	... 次のタグが無い
//------------------------------------------------
// 20050831 add (shiina, yatou)
YAO_STATUS YAO_NextTag( YAO_FILE * ioFile )
{
	YAO_STATUS 		rc; // return code
	YAO_METADATA	metaData;
	YAO_TAG 		currentTag;

	// 有効なYAOファイル？
	if( MyIsValidYAO_FILE( ioFile ) == false )
	{
		return kYAO_Failure;
	}
	
	// カレントがあるか？
	if( ioFile->currentExist == false )
	{
		return kYAO_NoCurrent;
	}

	// 現在タグの最初のメンバを取得
	currentTag = ioFile->currentMetaData.dataTag;
	metaData = ioFile->currentMetaData;
	if( IsTagObj( metaData ) == false ) // カレントが最初のメンバなら何もしない
	{
		rc = FindFirstMember( ioFile, currentTag, &metaData );
		if( rc != kYAO_Success )
		{
			return rc;
		}
	}
	
	// 次のタグオブジェクトを取得
	rc = FindNextTagObj( ioFile, &metaData );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	// 取得したタグオブジェクトをカレントに設定
	ioFile->currentMetaData = metaData;

	return kYAO_Success;
}

//------------------------------------------------
// 関数 : YAO_PreviousTag
// 機能 : 前のタグオブジェクトに移動
// 返値 : kYAO_Success		... 成功
//        kYAO_Failure		... 失敗
//        kYAO_FileIO_Err	... ファイル入出力エラー
//        kYAO_NoCurrent	... カレントが無い
//        kYAO_TagNotFound	... 前のタグが無い
//------------------------------------------------
// 20050831 add (shiina, yatou)
YAO_STATUS YAO_PreviousTag( YAO_FILE * ioFile )
{
	YAO_STATUS 		rc; // return code
	YAO_METADATA	metaData;
	YAO_TAG 		currentTag;

	// 有効なYAOファイル？
	if( MyIsValidYAO_FILE( ioFile ) == false )
	{
		return kYAO_Failure;
	}
	
	// カレントがあるか？
	if( ioFile->currentExist == false )
	{
		return kYAO_NoCurrent;
	}

	// 現在タグの最初のメンバを取得
	currentTag = ioFile->currentMetaData.dataTag;
	metaData = ioFile->currentMetaData;
	if( IsTagObj( metaData ) == false ) // カレントが最初のメンバなら何もしない
	{
		rc = FindFirstMember( ioFile, currentTag, &metaData );
		if( rc != kYAO_Success )
		{
			return rc;
		}
	}
	
	// 前のタグオブジェクトを取得
	rc = FindPreviousTagObj( ioFile, &metaData );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	// 前のタグがエントリーポイントであるか？
	if( metaData.dataTag == kEntryPointTag )
	{
		return kYAO_TagNotFound;
	}

	// 取得したタグオブジェクトをカレントに設定
	ioFile->currentMetaData = metaData;

	return kYAO_Success;
}

//------------------------------------------------
// 関数 : IsTagObj
// 機能 : タグオブジェクトかどうかを判定
// 返値 : true		... タグオブジェクトである
//        false		... タグオブジェクトでない
//------------------------------------------------
// 20050831 mod (shiina, yatou) エントリーポイントであるかどうかの判定は必要ないので削除
static bool IsTagObj( YAO_METADATA inMetaData )
{
	// 最初のメンバであるか？
	if( inMetaData.previousMemberPointer == kMemberNotFound )
	{
		return true;
	}
	
	return false;
}

//------------------------------------------------
// 関数 : InitMetaData
// 機能 : メタデータを初期化する
//------------------------------------------------
static void InitMetaData( YAO_METADATA * ioMetaDataPtr )
{
	ioMetaDataPtr->dataTag					= 0;				// タグを0に初期化
	ioMetaDataPtr->length					= 0;				// データの長さを0に初期化
	ioMetaDataPtr->filePointer				= 0;				// ファイル中の位置を0に初期化
	ioMetaDataPtr->previousMemberPointer	= kMemberNotFound;	// 前のメンバなし
	ioMetaDataPtr->nextMemberPointer		= kMemberNotFound;	// 次のメンバなし
	ioMetaDataPtr->previousTagPointer		= kTagNotFound;		// 前のタグなし
	ioMetaDataPtr->nextTagPointer			= kTagNotFound;		// 次のタグなし
}

//------------------------------------------------
// 関数 : ReadMetaData
// 機能 : メタデータをファイルから読み込む
// 返値 : kYAO_Success		... 成功
//        kYAO_FileIO_Err	... ファイル入出力エラー
//------------------------------------------------
// 20050831 add (shiina, yatou) 
static YAO_STATUS ReadMetaData( YAO_FILE * ioFile, long inPosition, YAO_METADATA * outMetaDataPtr )
{
	YAO_STATUS	rc; // return code
	
	// 与えられた位置に読み出し位置を移動
	rc = MySetFilePointer( ioFile, inPosition );
	if( rc != kYAO_Success )
	{
		return rc;
	}
	
	// メタデータをファイルから読み込む
	rc = MyReadData( ioFile, outMetaDataPtr, sizeof(YAO_METADATA) );
	if( rc != kYAO_Success )
	{
		return rc;
	}
	
	return kYAO_Success;
}

//------------------------------------------------
// 関数 : WriteMetaData
// 機能 : メタデータをファイルに書き込む
// 返値 : kYAO_Success		... 成功
//        kYAO_FileIO_Err	... ファイル入出力エラー
//------------------------------------------------
// 20050831 add (shiina, yatou)
static YAO_STATUS WriteMetaData( YAO_FILE * ioFile, YAO_METADATA * inMetaDataPtr )
{
	YAO_STATUS	rc; // return code
	
	// 指定されたメタデータのファイル上の位置に書き込み位置を移動
	rc = MySetFilePointer( ioFile, inMetaDataPtr->filePointer );
	if( rc != kYAO_Success )
	{
		return rc;
	}
	
	// 指定されたメタデータをファイルに書き込む
	rc = MyWriteData( ioFile, inMetaDataPtr, sizeof(YAO_METADATA) );
	if( rc != kYAO_Success )
	{
		return rc;
	}
	
	return kYAO_Success;
}

//------------------------------------------------
// 関数 : FindFirstTagObj
// 機能 : エントリーポイントを取得
// 返値 : kYAO_Success		... 成功
//        kYAO_Failure		... 失敗
//        kYAO_FileIO_Err	... ファイル入出力エラー
//------------------------------------------------
// 20050831 mod (shiina, yatou) 返り値をlongからYAO_STATUSに修正
static YAO_STATUS FindFirstTagObj( YAO_FILE * ioFile, YAO_METADATA * outMetaDataPtr )
{
	YAO_STATUS rc; // return code

	// 有効なYAOファイル？
	if( MyIsValidYAO_FILE( ioFile ) == false )
	{
		return kYAO_Failure;
	}
	
	// 出力先が指定されているか？
	if( outMetaDataPtr == NULL )
	{
		return kYAO_Failure;
	}

	// エントリーポイントをファイルから読み込む
	rc = ReadMetaData( ioFile, sizeof(YAO_FILE_HEADER), outMetaDataPtr );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	return kYAO_Success;
}


//------------------------------------------------
// 関数 : FindNextTagObj
// 機能 : 次のタグオブジェクトを取得
// 返値 : kYAO_Success		... 成功
//        kYAO_Failure		... 失敗
//        kYAO_FileIO_Err	... ファイル入出力エラー
//        kYAO_TagNotFound	... 見つからなかった
//------------------------------------------------
// 20050831 mod (shiina, yatou) 返り値をlongからYAO_STATUSに修正
static YAO_STATUS FindNextTagObj( YAO_FILE * ioFile, YAO_METADATA * ioMetaDataPtr )
{
	YAO_STATUS 	rc;	// return code
	long 		nextTagObjPos;

	// 有効なYAOファイル？
	if( MyIsValidYAO_FILE( ioFile ) == false )
	{
		return kYAO_Failure;
	}
	
	// メタデータが指定されているか？
	if( ioMetaDataPtr == NULL )
	{
		return kYAO_Failure;
	}

	// 指定されたメタデータがタグオブジェクトであるか？
	if( IsTagObj( *ioMetaDataPtr ) == false )
	{
		return kYAO_Failure;
	}
	
	// 指定されたメタデータの次にあるタグオブジェクトの位置を取得
	nextTagObjPos = ioMetaDataPtr->nextTagPointer;
	if( nextTagObjPos == kTagNotFound )
	{
		return kYAO_TagNotFound;
	}

	// タグオブジェクトをファイルから読み込む
	rc = ReadMetaData( ioFile, nextTagObjPos, ioMetaDataPtr );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	return kYAO_Success;
}


//------------------------------------------------
// 関数 : FindPreviousTagObj
// 機能 : 前のタグオブジェクトを取得
// 返値 : kYAO_Success		... 成功
//        kYAO_Failure		... 失敗
//        kYAO_FileIO_Err	... ファイル入出力エラー
//        kYAO_TagNotFound	... 見つからなかった
//------------------------------------------------
// 20050831 mod (shiina, yatou) 返り値をlongからYAO_STATUSに修正
static YAO_STATUS FindPreviousTagObj( YAO_FILE * ioFile, YAO_METADATA * ioMetaDataPtr )
{
	YAO_STATUS 	rc; // return code
	long 		prevTagObjPos;

	// 有効なYAOファイル？
	if( MyIsValidYAO_FILE( ioFile ) == false )
	{
		return kYAO_Failure;
	}
	
	// メタデータが指定されているか？
	if( ioMetaDataPtr == NULL )
	{
		return kYAO_Failure;
	}

	// 指定されたメタデータがタグオブジェクトであるか？
	if( IsTagObj( *ioMetaDataPtr ) == false )
	{
		return kYAO_Failure;
	}

	// 指定されたメタデータの前にあるタグオブジェクトの位置を取得
	prevTagObjPos = ioMetaDataPtr->previousTagPointer;
	if( prevTagObjPos == kTagNotFound )
	{
		return kYAO_TagNotFound;
	}
	
	// タグオブジェクトをファイルから読み込む
	rc = ReadMetaData( ioFile, prevTagObjPos, ioMetaDataPtr );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	return kYAO_Success;
}
		
//------------------------------------------------
// 関数 : FindLastTagObj
// 機能 : 最後のタグオブジェクトを取得
// 返値 : kYAO_Success		... 成功
//        kYAO_Failure		... 失敗
//        kYAO_FileIO_Err	... ファイル入出力エラー
//------------------------------------------------
// 20050831 mod (shiina, yatou) 返り値をlongからYAO_STATUSに修正
static YAO_STATUS FindLastTagObj( YAO_FILE * ioFile, YAO_METADATA * outMetaDataPtr )
{
	YAO_STATUS 		rc; // return code
	YAO_METADATA 	metaData;

	// 有効なYAOファイル？
	if( MyIsValidYAO_FILE( ioFile ) == false )
	{
		return kYAO_Failure;
	}
	
	// 出力先が指定されているか？
	if( outMetaDataPtr == NULL )
	{
		return kYAO_Failure;
	}
	

	// エントリーポイント取得
	rc = FindFirstTagObj( ioFile, &metaData );
	if( rc != kYAO_Success )
	{
		return rc;
	}
	
	// 最後のタグオブジェクトを検索
	while( metaData.nextTagPointer != kTagNotFound )
	{
		// 次のタグオブジェクトへ
		rc = FindNextTagObj( ioFile, &metaData );
		if( rc != kYAO_Success )
		{
			return rc;
		}
	}

	*outMetaDataPtr = metaData;
	return kYAO_Success;
}

//------------------------------------------------
// 関数 : FindFirstMember
// 機能 : 指定タグの最初のメンバを取得
// 返値 : kYAO_Success		... 成功
//        kYAO_Failure		... 失敗
//        kYAO_FileIO_Err	... ファイル入出力エラー
//        kYAO_TagNotFound	... 指定タグが見つからなかった
//------------------------------------------------
// 20050831 add (shiina, yatou)
static YAO_STATUS FindFirstMember( YAO_FILE * ioFile, YAO_TAG inTag, YAO_METADATA * outMetaDataPtr )
{
	YAO_STATUS		rc;	// return code
	YAO_METADATA	metaData;

	// 有効なYAOファイル？
	if( MyIsValidYAO_FILE( ioFile ) == false )
	{
		return kYAO_Failure;
	}

	// 出力先が指定されているか？
	if( outMetaDataPtr == NULL )
	{
		return kYAO_Failure;
	}

	// 入力タグがエントリーポイントのタグであればエラー
	if( inTag == kEntryPointTag )
	{
		return kYAO_Failure;
	}

	// エントリーポイント取得
	rc = FindFirstTagObj( ioFile, &metaData );
	if( rc != kYAO_Success )
	{
		return rc;
	}
	
	// 指定タグを持つタグオブジェクトを検索
	while( metaData.dataTag != inTag )
	{
		// 次のタグオブジェクトへ
		rc = FindNextTagObj( ioFile, &metaData );
		if( rc != kYAO_Success )
		{
			return rc;
		}
	}

	// 出力
	*outMetaDataPtr = metaData;
	return kYAO_Success;
}

//------------------------------------------------
// 関数 : FindNextMember
// 機能 : 指定メタデータの次のメタデータを取得
// 返値 : kYAO_Success			... 成功
//        kYAO_Failure			... 失敗
//        kYAO_FileIO_Err		... ファイル入出力エラー
//        kYAO_MemberNotFound	... 次のメンバが見つからなかった
//------------------------------------------------
// 20050831 add (shiina, yatou)
static YAO_STATUS FindNextMember( YAO_FILE * ioFile, YAO_METADATA * ioMetaDataPtr )
{
	YAO_STATUS	rc;	// return code

	// 与えられたメタデータの次にメンバが存在するか？
	if( ioMetaDataPtr->nextMemberPointer == kMemberNotFound )
	{
		return kYAO_MemberNotFound;
	}

	// 次のメンバを読み出す
	rc = ReadMetaData( ioFile, ioMetaDataPtr->nextMemberPointer, ioMetaDataPtr );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	return kYAO_Success;
}

//------------------------------------------------
// 関数 : FindPreviousMember
// 機能 : 指定メタデータの前のメタデータを取得
// 返値 : kYAO_Success			... 成功
//        kYAO_Failure			... 失敗
//        kYAO_FileIO_Err		... ファイル入出力エラー
//        kYAO_MemberNotFound	... 前のメンバが見つからなかった
//------------------------------------------------
// 20050831 add (shiina, yatou)
static YAO_STATUS FindPreviousMember( YAO_FILE * ioFile, YAO_METADATA * ioMetaDataPtr )
{
	YAO_STATUS	rc;	// return code

	// 与えられたメタデータの前にメンバが存在するか？
	if( ioMetaDataPtr->previousMemberPointer == kMemberNotFound )
	{
		return kYAO_MemberNotFound;
	}

	// 前のメンバを読み出す
	rc = ReadMetaData( ioFile, ioMetaDataPtr->previousMemberPointer, ioMetaDataPtr );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	return kYAO_Success;
}

//------------------------------------------------
// 関数 : FindLastMember
// 機能 : 指定タグの最後のメンバを取得
// 返値 : kYAO_Success			... 成功
//        kYAO_Failure			... 失敗
//        kYAO_FileIO_Err		... ファイル入出力エラー
//        kYAO_TagNotFound		... 指定タグが見つからなかった
//------------------------------------------------
// 20050831 mod (shiina, yatou) 返り値をlongからYAO_STATUSに修正
static YAO_STATUS FindLastMember( YAO_FILE * ioFile, YAO_TAG inTag, YAO_METADATA * outMetaDataPtr )
{
	YAO_STATUS		rc; // return code;
	YAO_METADATA	metaData;

	// 有効なYAOファイル？
	if( MyIsValidYAO_FILE( ioFile ) == false )
	{
		return kYAO_Failure;
	}
	
	// 出力先が指定されているか？
	if( outMetaDataPtr == NULL )
	{
		return kYAO_Failure;
	}

	// 指定タグの先頭のメンバを取得
	rc = FindFirstMember( ioFile, inTag, &metaData );
	if( rc != kYAO_Success )
	{
		return rc;
	}
	
	// 最後のメンバを検索
	while( metaData.nextMemberPointer != kMemberNotFound )
	{
		// 次のメンバへ
		rc = FindNextMember( ioFile, &metaData );
		if( rc != kYAO_Success )
		{
			return rc;
		}
	}

	// 出力
	*outMetaDataPtr = metaData;
	return kYAO_Success;
}

//------------------------------------------------
// 関数 : AddTagObj
// 機能 : タグオブジェクトを追加
// 返値 : kYAO_Success    ... 成功
//        kYAO_Failure    ... 失敗
//        kYAO_FileIO_Err ... ファイル入出力エラー
//------------------------------------------------
// 20050831 add (shiina , yatou)
static YAO_STATUS AddTagObj( YAO_FILE * ioFile, YAO_TAG inTag, long inDataLen, void * inDataBuff )
{
	YAO_STATUS 		rc; // return code
	YAO_METADATA	newMetaData;
	YAO_METADATA	lastTagObj;

	// 最後のタグオブジェクトを取得
	rc = FindLastTagObj( ioFile, &lastTagObj );
	if( rc != kYAO_Success )
	{
		return rc;
	}
	
	// 追加するメタデータを初期化
	InitMetaData( &newMetaData );

	// ファイルの末尾に書き込み位置を移動
	rc = MySetFilePointerToEnd( ioFile );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	// 最後のタグオブジェクトの次の位置をファイルの末尾とする
	lastTagObj.nextTagPointer = MyGetCurrentFilePointer( ioFile );

	// 最後のタグオブジェクトをファイルに書き込む
	rc = WriteMetaData( ioFile, &lastTagObj );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	// ファイルの末尾に書き込み位置を移動
	rc = MySetFilePointerToEnd( ioFile );
	if( rc != kYAO_Success )
	{
		return rc;
	}
	
	// 追加するメタデータを作る
	newMetaData.dataTag = inTag;
	newMetaData.length = inDataLen;
	newMetaData.previousTagPointer = lastTagObj.filePointer;
	newMetaData.filePointer = MyGetCurrentFilePointer( ioFile ); // ファイルの末尾を指定
	
	// 追加するメタデータをファイルに書き込む
	rc = MyWriteData( ioFile, &newMetaData, sizeof(YAO_METADATA) );
	if( rc != kYAO_Success )
	{
		return rc;
	}
	
	// 追加する実データをファイルに書き込む
	rc = MyWriteData( ioFile, inDataBuff, inDataLen );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	return kYAO_Success;
}

//------------------------------------------------
// 関数 : AddMemberObj
// 機能 : メンバを追加
// 返値 : kYAO_Success		... 成功
//        kYAO_Failure		... 失敗
//        kYAO_FileIO_Err	... ファイル入出力エラー
//        kYAO_TagNotFound	... 指定タグのメンバが存在しない
//------------------------------------------------
// 20050831 add (shiina , yatou)
static YAO_STATUS AddMemberObj( YAO_FILE * ioFile, YAO_TAG inTag, long inDataLen, void * inDataBuff )
{
	YAO_STATUS 		rc; // return code
	YAO_METADATA 	lastMember;
	YAO_METADATA 	newMetaData;
	
	// 指定タグの最後のメンバを取得
	rc = FindLastMember( ioFile, inTag, &lastMember );
	if( rc != kYAO_Success )
	{
		return rc;
	}
	
	// 追加するメタデータを初期化
	InitMetaData( &newMetaData );
	
	// ファイルの末尾に書き込み位置を移動
	rc = MySetFilePointerToEnd( ioFile );
	if( rc != kYAO_Success )
	{	
		return rc;
	}

	// 最後のメンバの次の位置をファイルの末尾とする
	lastMember.nextMemberPointer = MyGetCurrentFilePointer( ioFile );
	
	// 変更した最後のメタデータをファイルに書き込む
	rc = WriteMetaData( ioFile, &lastMember );
	if( rc != kYAO_Success )
	{
		return rc;
	}
	
	// ファイルの末尾に書き込み位置を移動
	rc = MySetFilePointerToEnd( ioFile );
	if( rc != kYAO_Success )
	{
		return rc;
	}
	
	// 追加するメタデータを作る
	newMetaData.dataTag = inTag;
	newMetaData.length = inDataLen;
	newMetaData.previousMemberPointer = lastMember.filePointer;
	newMetaData.filePointer = MyGetCurrentFilePointer( ioFile ); // ファイルの末尾を指定
	
	// 追加するメタデータをファイルに書き込む
	rc = MyWriteData( ioFile, &newMetaData, sizeof(YAO_METADATA) );
	if( rc != kYAO_Success )
	{
		return rc;
	}
	
	// 追加する実データを書き込む
	rc = MyWriteData( ioFile, inDataBuff, inDataLen );
	if( rc != kYAO_Success )
	{
		return rc;
	}
	
	return kYAO_Success;
}

//------------------------------------------------
// 関数 : RemoveTagObj
// 機能 : タグオブジェクトを削除
// 返値 : kYAO_Success    ... 成功
//        kYAO_Failure    ... 失敗
//        kYAO_FileIO_Err ... ファイル入出力エラー
//------------------------------------------------
// 20050831 mod (shiina, yatou) カレントのタグオブジェクトを削除するように変更
static YAO_STATUS RemoveTagObj( YAO_FILE * ioFile )
{
	YAO_STATUS		rc; // return code
	long			previousTagPos;
	long			nextTagPos;
	long			nextMemberPos;
	
	// カレントの前タグと次タグの位置、カレントの次メンバの位置を取得
	previousTagPos = ioFile->currentMetaData.previousTagPointer;
	nextTagPos = ioFile->currentMetaData.nextTagPointer;
	nextMemberPos = ioFile->currentMetaData.nextMemberPointer;

	// 同じタグのメンバがタグオブジェクト以外に無いか？
	if( nextMemberPos == kMemberNotFound )
	{
		// カレントの前にあるタグオブジェクトが持つ、次のタグへのポインタを書き換える
		rc = UpdatePreviousTag( ioFile, nextTagPos );
		if( rc != kYAO_Success )
		{
			return rc;
		}
		
		if( nextTagPos != kTagNotFound ) // 次のタグが無ければ何もしない
		{
			// カレントの次にあるタグオブジェクトが持つ、前のタグへのポインタを書き換える
			rc = UpdateNextTag( ioFile, previousTagPos );
			if( rc != kYAO_Success )
			{
				return rc;
			}
		}
	}
	// タグオブジェクト以外にもメンバがある場合
	else
	{
		// カレントの前にあるタグオブジェクトが持つ、次のタグへのポインタを書き換える
		rc = UpdatePreviousTag( ioFile, nextMemberPos ); // 次のメンバ
		if( rc != kYAO_Success )
		{
			return rc;
		}
		
		if( nextTagPos != kTagNotFound ) // 次のタグが無ければ何もしない
		{
			// カレントの次にあるタグオブジェクトが持つ、前のタグへのポインタを書き換える
			rc = UpdateNextTag( ioFile, nextMemberPos ); // 次のメンバ
			if( rc != kYAO_Success )
			{
				return rc;
			}
		}
		
		// カレントの次のメンバが持つ、前タグと次タグへのポインタを書き換える
		rc = UpdateNextMember( ioFile, previousTagPos, nextTagPos );
		if( rc != kYAO_Success )
		{
			return rc;
		}
		
	}
	
	// カレントは削除されたので存在しない
	ioFile->currentExist = false;
	
	return kYAO_Success;
}

//------------------------------------------------
// 関数 : RemoveMemberObj
// 機能 : メンバを削除
// 返値 : kYAO_Success    ... 成功
//        kYAO_Failure    ... 失敗
//        kYAO_FileIO_Err ... ファイル入出力エラー
//------------------------------------------------
// 20050831 mod (shiina, yatou) カレントのメンバを削除するように変更
static YAO_STATUS RemoveMemberObj( YAO_FILE * ioFile )
{
	YAO_STATUS		rc;	// return code
	YAO_METADATA	previousMemberObj;
	YAO_METADATA	nextMemberObj;
	long			nextMemberObjPos;
	long			prevMemberObjPos;

	// カレントの前のメンバを取得
	previousMemberObj = ioFile->currentMetaData;
	rc = FindPreviousMember( ioFile, &previousMemberObj );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	// 前のメンバが持つ次メンバへのポインタを、カレントの次メンバに設定
	nextMemberObjPos = ioFile->currentMetaData.nextMemberPointer;
	previousMemberObj.nextMemberPointer = nextMemberObjPos;

	// 前のメンバをファイルに書き込む
	rc = WriteMetaData( ioFile, &previousMemberObj );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	// カレントの次のメンバを取得
	nextMemberObj = ioFile->currentMetaData;
	rc = FindNextMember( ioFile, &nextMemberObj );
	if( rc == kYAO_Success ) // 次のメンバが無ければ何もしない
	{
		// 次のメンバが持つ前メンバへのポインタを、カレントの前メンバに設定
		prevMemberObjPos = ioFile->currentMetaData.previousMemberPointer;
		nextMemberObj.previousMemberPointer = prevMemberObjPos;

		// 次のメンバをファイルに書き込む
		rc = WriteMetaData( ioFile, &nextMemberObj );
		if( rc != kYAO_Success )
		{
			return rc;
		}
	}

	// カレントは削除されたので存在しない
	ioFile->currentExist = false;

	return kYAO_Success;
}

//------------------------------------------------
// 関数 : InsertTagObj
// 機能 : タグオブジェクトを挿入
// 返値 : kYAO_Success    ... 成功
//        kYAO_Failure    ... 失敗
//        kYAO_FileIO_Err ... ファイル入出力エラー
//------------------------------------------------
// 20050831 mod (shiina, yatou) カレントの位置にデータを挿入するように変更
static YAO_STATUS InsertTagObj( YAO_FILE * ioFile, long inDataLen, void * inDataBuff )
{
	YAO_STATUS		rc;	// return code
	
	YAO_METADATA	newMetaData;
	YAO_METADATA	prevTagMetaData;
	YAO_METADATA	nextTagMetaData;
	
	// ファイルの末尾に書き込み位置を移動
	rc = MySetFilePointerToEnd( ioFile );
	if( rc != kYAO_Success )
	{
		return rc;
	}
	
	// 新しく挿入するメタデータを作成
	InitMetaData( &newMetaData );
	newMetaData.dataTag = ioFile->currentMetaData.dataTag;			// カレントと同じタグを指定
	newMetaData.filePointer = MyGetCurrentFilePointer( ioFile );	// ファイルの末尾を指定
	newMetaData.length = inDataLen;
	
	// カレントの前のタグオブジェクトを取得
	prevTagMetaData = ioFile->currentMetaData;
	rc = FindPreviousTagObj( ioFile, &prevTagMetaData );
	if( rc != kYAO_Success )
	{
		return rc;
	}
	
	// 前のタグオブジェクトに、新しく挿入するタグオブジェクトを連結
	prevTagMetaData.nextTagPointer = newMetaData.filePointer;
	newMetaData.previousTagPointer = prevTagMetaData.filePointer;

	// 前のタグオブジェクトをファイルに書き込む
	rc = WriteMetaData( ioFile, &prevTagMetaData );
	if( rc != kYAO_Success )
	{
		return rc;
	}
	
	// 次のタグオブジェクトを取得
	nextTagMetaData = ioFile->currentMetaData;
	rc = FindNextTagObj( ioFile, &nextTagMetaData );
	if( rc == kYAO_Success ) // 次のタグが無いなら何もしない
	{
		// 次のタグオブジェクトに、新しく挿入するタグオブジェクトを連結
		nextTagMetaData.previousTagPointer = newMetaData.filePointer;
		newMetaData.nextTagPointer = nextTagMetaData.filePointer;

		// 次のタグオブジェクトをファイルに書き込む
		rc = WriteMetaData( ioFile, &nextTagMetaData );
		if( rc != kYAO_Success )
		{
			return rc;
		}
	}
	
	// カレントの前に、新しいタグオブジェクトを挿入する
	ioFile->currentMetaData.previousTagPointer = kTagNotFound;
	ioFile->currentMetaData.nextTagPointer = kTagNotFound;
	ioFile->currentMetaData.previousMemberPointer = newMetaData.filePointer;
	newMetaData.nextMemberPointer = ioFile->currentMetaData.filePointer;
	
	// 更新したカレントをファイルに書き込む
	rc = WriteMetaData( ioFile, &(ioFile->currentMetaData) );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	// 新しいメタデータをファイルに書き込む
	rc = WriteMetaData( ioFile, &newMetaData );
	if( rc != kYAO_Success )
	{
		return rc;
	}
	
	// 新しく挿入する実データを書き込む
	rc = MyWriteData( ioFile, inDataBuff, inDataLen );
	if( rc != kYAO_Success )
	{
		return kYAO_FileIO_Err;
	}
	
	return kYAO_Success;
}


//------------------------------------------------
// 関数 : InsertMemberObj
// 機能 : メンバを挿入
// 返値 : kYAO_Success    ... 成功
//        kYAO_Failure    ... 失敗
//        kYAO_FileIO_Err ... ファイル入出力エラー
//------------------------------------------------
// 20050831 mod (shiina, yatou) カレントの位置にデータを挿入するように変更
static YAO_STATUS InsertMemberObj( YAO_FILE * ioFile, long inDataLen, void * inDataBuff )
{
	YAO_STATUS		rc;	// return code
	YAO_METADATA	newMetaData;
	YAO_METADATA	prevMember;
	
	// ファイルの末尾に書き込み位置を移動
	rc = MySetFilePointerToEnd( ioFile );
	if( rc != kYAO_Success )
	{
		return rc;
	}
	
	// 新しく挿入するメタデータを作成
	InitMetaData( &newMetaData );
	newMetaData.dataTag = ioFile->currentMetaData.dataTag;
	newMetaData.filePointer = MyGetCurrentFilePointer( ioFile ); // ファイルの末尾を指定
	newMetaData.length = inDataLen;
	
	// カレントの前のメンバを取得
	prevMember = ioFile->currentMetaData;
	rc = FindPreviousMember( ioFile, &prevMember );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	// 前のメンバに、新しく挿入するメンバを連結
	prevMember.nextMemberPointer = newMetaData.filePointer;
	newMetaData.previousMemberPointer = prevMember.filePointer;

	// 前のメンバをファイルに書き込む
	rc = WriteMetaData( ioFile, &prevMember );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	// カレントの前に新しいメンバを挿入
	ioFile->currentMetaData.previousMemberPointer = newMetaData.filePointer;
	newMetaData.nextMemberPointer = ioFile->currentMetaData.filePointer;

	// 更新したカレントをファイルに書き込む
	rc = WriteMetaData( ioFile, &(ioFile->currentMetaData) );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	// 新しいメタデータをファイルに書き込む
	rc = WriteMetaData( ioFile, &newMetaData );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	// 新しく挿入する実データを書き込む
	rc = MyWriteData( ioFile, inDataBuff, inDataLen );
	if( rc != kYAO_Success )
	{
		return rc;
	}
	
	return kYAO_Success;
}

//------------------------------------------------
// 関数 : UpdateNextTag
// 機能 : カレントの次に来るタグオブジェクトがもつ前タグへのポインタを変更
// 返値 : kYAO_Success    ... 成功
//        kYAO_Failure    ... 失敗
//        kYAO_FileIO_Err ... ファイル入出力エラー
//------------------------------------------------
// 20050901 add (shiina, yatou) RemoveTagObjでのみ使われる
static YAO_STATUS UpdateNextTag( YAO_FILE * ioFile, long inPreviousTagPos )
{
	YAO_STATUS		rc; // return code
	YAO_METADATA	nextTagObj;
	
	// カレントの次のタグオブジェクトを取得
	nextTagObj = ioFile->currentMetaData;
	rc = FindNextTagObj( ioFile, &nextTagObj );
	if( rc == kYAO_TagNotFound ) // 無い場合は失敗
	{
		return kYAO_Failure;
	}
	if( rc != kYAO_Success ) // 次のタグがあるが、取得に失敗したか？
	{
		return rc;
	}
	
	// 次のタグが持つ、前のタグへのポインタを書き換え
	nextTagObj.previousTagPointer = inPreviousTagPos;
	
	// 更新したタグオブジェクトをファイルに書き込む
	rc = WriteMetaData( ioFile, &nextTagObj );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	return rc;
}

//------------------------------------------------
// 関数 : UpdatePreviousTag
// 機能 : カレントの前に来るタグオブジェクトがもつ次タグへのポインタを変更
// 返値 : kYAO_Success    ... 成功
//        kYAO_Failure    ... 失敗
//        kYAO_FileIO_Err ... ファイル入出力エラー
//------------------------------------------------
// 20050901 add (shiina, yatou) RemoveTagObjでのみ使われる
static YAO_STATUS UpdatePreviousTag( YAO_FILE * ioFile, long inNextTagPos )
{
	YAO_STATUS		rc; // return code
	YAO_METADATA	previousTagObj;
	
	// カレントの前のタグオブジェクトを取得
	previousTagObj = ioFile->currentMetaData;
	rc = FindPreviousTagObj( ioFile, &previousTagObj );
	// 無い場合は失敗
	if( rc == kYAO_TagNotFound )
	{
		return kYAO_Failure;
	}
	// 前のタグがあるが、取得に失敗したか？
	if( rc != kYAO_Success )
	{
		return rc;
	}
	
	// 前のタグが持つ、次のタグへのポインタを書き換え
	previousTagObj.nextTagPointer = inNextTagPos;
	
	// 更新したタグオブジェクトをファイルに書き込む
	rc = WriteMetaData( ioFile, &previousTagObj );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	return rc;
}

//------------------------------------------------
// 関数 : UpdateNextMember
// 機能 : カレントの次に来るメンバがもつ、前タグおよび次タグへのポインタを変更
// 返値 : kYAO_Success    ... 成功
//        kYAO_Failure    ... 失敗
//        kYAO_FileIO_Err ... ファイル入出力エラー
//------------------------------------------------
// 20050901 add (shiina, yatou) RemoveTagObjでのみ使われる
static YAO_STATUS UpdateNextMember( YAO_FILE * ioFile, long inPreviousTagPos, long inNextTagPos )
{
	YAO_STATUS		rc; // return code
	YAO_METADATA	nextMember;
	
	// カレントの次のメンバを取得
	nextMember = ioFile->currentMetaData;
	rc = FindNextMember( ioFile, &nextMember );
	// 無い場合は失敗
	if( rc == kYAO_MemberNotFound )
	{
		return kYAO_Failure;
	}
	// 次のメンバがあるが、取得に失敗したか？
	if( rc != kYAO_Success )
	{
		return rc;
	}
	
	// 次のメンバを先頭メンバにする
	nextMember.previousMemberPointer = kMemberNotFound;
	// カレントの次にあるメンバの、前タグと次タグの位置を設定
	nextMember.previousTagPointer = inPreviousTagPos;
	nextMember.nextTagPointer = inNextTagPos;
	
	// カレントの次のメンバをファイルに書き込む
	rc = WriteMetaData( ioFile, &nextMember );
	if( rc != kYAO_Success )
	{
		return rc;
	}

	return rc;
}



//------------------------------------------------
// 関数 : MyIsFileExists
// 機能 : 指定ファイルが存在するか調べる
// 返値 : true	... 存在する
//        false	... 存在しない   
//------------------------------------------------
// 20050831 add (shiina, yatou)  ファイルの存在はfopenで調べるしかない
static bool MyIsFileExists( const char * inFilePath )
{
	FILE * fp;
	
	// ファイルを読み込みモードで試しに開く
	fp = fopen( inFilePath, "r" );
	if( fp != NULL )
	{
		// もしファイルが存在すれば閉じてからtrueを返す
		fclose( fp );
		return true;
	}
	
	// ファイルが存在しない
	return false;
}

//------------------------------------------------
// 関数 : MySetFilePointer
// 機能 : ファイルポインタを指定位置に移動する
// 返値 : kYAO_Success		... 成功
//        kYAO_FileIO_Err	... ファイル入出力エラー
//------------------------------------------------
// 20050831 mod (shiina, yatou) Windows非依存化
static YAO_STATUS MySetFilePointer( YAO_FILE * ioFile, long inPosition )
{
	int seekResult; // fseekの戻り値 (0以外なら失敗)
	
	// 有効なYAOファイル？
	if( MyIsValidYAO_FILE( ioFile ) == false )
	{
		return kYAO_FileIO_Err;
	}

	// ファイルポインタを移動
	seekResult = fseek( ioFile->filePointer, inPosition, SEEK_SET );
	if( seekResult != 0 )
	{
		return kYAO_FileIO_Err;
	}
	
	return kYAO_Success;
}

//------------------------------------------------
// 関数 : MySetFilePointerToEnd
// 機能 : ファイルポインタをファイルの最後に移動する
// 返値 : kYAO_Success		... 成功
//        kYAO_FileIO_Err	... ファイル入出力エラー
//------------------------------------------------
// 20050831 mod (shiina, yatou) Windows非依存化
static YAO_STATUS MySetFilePointerToEnd( YAO_FILE * ioFile )
{
	int seekResult; // fseekの戻り値 (0以外なら失敗)
	
	// 有効なYAOファイル？
	if( MyIsValidYAO_FILE( ioFile ) == false )
	{
		return kYAO_FileIO_Err;
	}

	// ファイルポインタの位置をファイルの末尾に移動
	seekResult = fseek( ioFile->filePointer, 0, SEEK_END );
	if( seekResult != 0 )
	{
		return kYAO_FileIO_Err;
	}
	
	return kYAO_Success;
}


//------------------------------------------------
// 関数 : MyGetCurrentFilePointer
// 機能 : ファイルポインタの現在位置を取得する
// 返値 : ファイルポインタの、ファイル先頭からのオフセット位置
//        失敗すると -1 が返る
//------------------------------------------------
// 20050831 mod (shiina, yatou) Windows非依存化
static long MyGetCurrentFilePointer( YAO_FILE * ioFile )
{
	long position;
	
	// 有効なYAOファイル？
	if( MyIsValidYAO_FILE( ioFile ) == false )
	{
		return -1;
	}

	position = ftell( ioFile->filePointer );
	return position;
}

//------------------------------------------------
// 関数 : MyReadData
// 機能 : ファイルからデータを指定バイト長だけ読み出す
// 返値 : kYAO_Success		... 成功
//        kYAO_FileIO_Err	... ファイル入出力エラー
//------------------------------------------------
// 20050831 mod (shiina, yatou) Windows非依存化
static YAO_STATUS MyReadData( YAO_FILE * inFile, void * outBuff, long inBufLen )
{
	long readActual;

	readActual = (long)fread( outBuff, 1, inBufLen, inFile->filePointer );
	
	// 指定した長さだけ読み込めなければ失敗
	if( readActual < inBufLen )
	{
		return kYAO_FileIO_Err;
	}
	
	return kYAO_Success;
}

//------------------------------------------------
// 関数 : MyWriteData
// 機能 : ファイルにデータを指定バイト長だけ書き込む
// 返値 : kYAO_Success		... 成功
//        kYAO_FileIO_Err	... ファイル入出力エラー
//------------------------------------------------
// 20050831 mod (shiina, yatou) Windows非依存化
static YAO_STATUS MyWriteData( YAO_FILE * inFile, const void * inBuff, long inBufLen )
{
	long wroteActual;

	wroteActual = (long)fwrite( inBuff, 1, inBufLen, inFile->filePointer );
	
	// 指定した長さだけ書き込めなければ失敗
	if( wroteActual < inBufLen )
	{
		return kYAO_FileIO_Err;
	}
	
	return kYAO_Success;
}

//------------------------------------------------
// 関数 : MyIsValidYAO_FILE
// 機能 : YAOファイルが有効かどうか確認する
// 返値 : true		... 有効
//        false		... 有効でない
//------------------------------------------------
// 20050831 add (shiina, yatou) 
static bool MyIsValidYAO_FILE( YAO_FILE * ioFile )
{
	// YAOファイルが指定されていないなら有効でない
	if( ioFile == NULL )
	{
		return false;
	}
	
	// ファイルが開かれていないなら有効でない
	if( ioFile->filePointer == NULL )
	{
		return false;
	}

	return true;
}