// TYPE: C
// PRJ: YAO
// PGM: YAO
// VER: 1.3.1
// USES: Visual C++ 6.0
// CREATE: 20040126 (iwai)
// UPDATE: 20041216 (yamamoto, sugawara)
// UPDATE: 20050831 (shiina, yatou)
// UPDATE: 20060228 (yamamoto)
// UPDATE: 20060907 (asami)
//////////////////////////////////////////////////
// 20041216 version 1.1.0 STL廃止。Cのラッパー廃止。全面的にCで書き直し。
// 20050624 version 1.2.1 Windows依存部分を標準ライブラリ関数を用いて書き直し。
//                        現在のメタデータ(通称「カレント」)をYAO_FILEに保持しながら
//                        操作するように仕様変更。アクセスモードを指定できるようになった。
// 20060228 version 1.3.0 YAO_ReplaceData 追加
// 20060907 version 1.3.1 YAOStatusをYAO_STATUSに,YAOTagをYAO_TAGに変更

#ifndef __YAO_H__
#define __YAO_H__

#include <stdio.h> 
#include <stdlib.h>
#include <string.h>

//**************************************
// 定数・型
//**************************************

#define kYAOVersion					 1	// YAO のバージョン

// データ型
#define YAO_STATUS	long				// YAO のエラー型
#define YAO_TAG		long				// YAO のタグを表す


#define kMemberNotFound (-1)			// メンバへのポインタの初期値。
#define kTagNotFound	(-1)			// タグオブジェクトへのポインタの初期値。
#define kEntryPointTag	(-1)			// エントリーポイント用タグ。

// YAO のアクセスモード
#define kYAO_ReadWrite				 0  // 読み書き両用
#define kYAO_ReadOnly				 1  // 読み取り専用

// 返り値
#define kYAO_Success				 0	// 成功
#define kYAO_Failure				-1	// 失敗
#define kYAO_FileIO_Err				-2  // ファイル入出力のエラー
#define kYAO_NoCurrent				-3  // カレントが無い
#define kYAO_MemberNotFound			-4  // メンバが見つからない
#define kYAO_TagNotFound			-5  // タグオブジェクトが見つからない
#define kYAO_FileIsReadOnly			-6  // 読み取り専用モードで書き込みを行おうとした


// YAO に格納されたデータに関する情報
typedef struct YAO_METADATA
{
	YAO_TAG	dataTag;				// このデータが属するデータ群のタグ(識別子)
	long	length;					// このデータの長さ
	long	filePointer;			// このデータのファイル先頭からのオフセット値
	long	previousMemberPointer;	// 前のメンバの格納位置
	long	nextMemberPointer;		// 次のメンバの格納位置
	long	previousTagPointer;		// 前のタグオブジェクトの格納位置
	long	nextTagPointer;			// 次のタグオブジェクトの格納位置
	char	reserved[8];			// 拡張用の予約
} YAO_METADATA;


// YAO ファイルの情報を格納した構造体
typedef struct YAO_FILE
{
	FILE* filePointer;
	YAO_METADATA currentMetaData;	// カレント
	bool currentExist;				// カレントが存在するか
	long accessMode;				// YAO のアクセスモード		
} YAO_FILE;


// YAO ファイルのヘッダ (512 Bytes)
typedef struct YAO_FILE_HEADER
{
	char	signature[4];		// "YAO"の文字
	short	version;			// YAO ファイルのバージョン
	char	reserved[506];		// 拡張用の予約
} YAO_FILE_HEADER;






//**************************************
// 関数
//**************************************

//------------------------------------------------
// 関数 : YAO_Create
// 機能 : YAO ファイルを新しく作成して開く
// 返値 : kYAO_Success			... 成功
//        kYAO_Failure			... 失敗
//        kYAO_FileIO_Err		... ファイル入出力エラー
//------------------------------------------------
// 20050912 mod (shiina, yatou) fopenを使用するよう変更
//                              YAO_FILEを返り値から引数に変更
//                              返り値をYAO_STATUSに変更
YAO_STATUS YAO_Create( const char * inFilePath, YAO_FILE * outFile );

//------------------------------------------------
// 関数 : YAO_Open
// 機能 : YAO ファイルを開く
// 返値 : kYAO_Success			... 成功
//        kYAO_Failure			... 失敗
//        kYAO_FileIO_Err		... ファイル入出力エラー
//------------------------------------------------
// 20050912 mod (shiina, yatou) fopenを使用するよう変更
//                              YAO_FILEを返り値から引数に変更
//                              アクセスモードを指定できるように変更
//                              返り値をYAO_STATUSに変更
YAO_STATUS YAO_Open( const char * inFilePath, long inAccessMode, YAO_FILE * outFile );

//-------------------------------------------------
// 名前 : YAO_Close
// 機能 : YAO ファイルを閉じる
//-------------------------------------------------
// 20050912 mod (shiina, yatou) fcloseを使用するよう変更
//                              YAO_FILEの領域開放を行わないようにした
void YAO_Close( YAO_FILE * ioFile );

//------------------------------------------------
// 関数 : YAO_AddData
// 機能 : 指定タグにデータを追加する
// 返値 : kYAO_Success			... 成功
//        kYAO_Failure			... 失敗
//        kYAO_FileIO_Err		... ファイル入出力エラー
//        kYAO_FileIsReadOnly	... 読み取り専用モード
//------------------------------------------------
// 20050912 mod (shiina, yatou) 内部処理をAddMemberObjとAddTagObjに分割
//                              引数inDataBuffを最後に渡すように変更
//                              読み取り専用モードならば直ちに終了するように変更
YAO_STATUS YAO_AddData( YAO_FILE * ioFile, YAO_TAG inTag, long inDataLen, void * inDataBuff );

//------------------------------------------------
// 関数 : YAO_InsertData
// 機能 : データをカレントの前に挿入する
// 返値 : kYAO_Success			... 成功
//        kYAO_Failure			... 失敗
//        kYAO_FileIO_Err		... ファイル入出力エラー
//        kYAO_NoCurrent		... カレントが無い
//        kYAO_FileIsReadOnly	... 読み取り専用モード
//------------------------------------------------
// 20050912 mod (shiina, yatou) カレントの前の位置にデータを挿入するように変更
//                              引数inDataBuffを最後に渡すように変更
//                              読み取り専用モードならば直ちに終了するように変更
YAO_STATUS YAO_InsertData( YAO_FILE * ioFile, long inDataLen, void * inDataBuff );

//------------------------------------------------
// 関数 : YAO_RemoveData
// 機能 : カレントの位置にあるデータを削除する
// 返値 : kYAO_Success			... 成功
//        kYAO_Failure			... 失敗
//        kYAO_FileIO_Err		... ファイル入出力エラー
//        kYAO_NoCurrent		... カレントが無い
//        kYAO_FileIsReadOnly	... 読み取り専用モード
//------------------------------------------------
// 20050912 mod (shiina, yatou) カレントのデータを削除するように変更
//                              読み取り専用モードならば直ちに終了するように変更
YAO_STATUS YAO_RemoveData( YAO_FILE * ioFile );

//------------------------------------------------
// 関数 : YAO_ReplaceData
// 機能 : カレントの位置にあるデータを置き換える
// 返値 : kYAO_Success			... 成功
//        kYAO_Failure			... 失敗
//        kYAO_FileIO_Err		... ファイル入出力エラー
//        kYAO_NoCurrent		... カレントが無い
//        kYAO_FileIsReadOnly	... 読み取り専用モード
//------------------------------------------------
// 20060228 add (yamamoto) 実装
YAO_STATUS YAO_ReplaceData( YAO_FILE * ioFile, long inDataLen, void * inDataBuff );

//------------------------------------------------
// 関数 : YAO_GetData
// 機能 : カレントのデータを取得する
// 返値 : kYAO_Success		... 成功
//        kYAO_Failure		... 失敗
//        kYAO_FileIO_Err	... ファイル入出力エラー
//        kYAO_NoCurrent	... カレントが無い
//------------------------------------------------
// 20050831 mod (shiina, yatou) カレントのデータを取得するように変更
YAO_STATUS YAO_GetData( YAO_FILE * ioFile, long inBufferLen, void * outBuff );

//------------------------------------------------
// 関数 : YAO_GetDataLength
// 機能 : カレントのデータの長さを取得
// 返値 : カレントのデータの長さが返る
//        エラー時やカレントが無いと -1 が返る
//------------------------------------------------
// 20050831 mod (shiina, yatou) カレントのデータの長さを取得するように変更
long YAO_GetDataLength( YAO_FILE * ioFile );

//------------------------------------------------
// 関数 : YAO_FirstMember
// 機能 : 指定タグの最初のメンバに移動
// 返値 : kYAO_Success		... 成功
//        kYAO_Failure		... 失敗
//        kYAO_FileIO_Err	... ファイル入出力エラー
//        kYAO_TagNotFound	... 指定タグが見つからなかった
//------------------------------------------------
// 20050831 mod (shiina, yatou) 内部関数FindFirstMemberを使用するように変更
YAO_STATUS YAO_FirstMember( YAO_FILE * ioFile, YAO_STATUS inTag );

//------------------------------------------------
// 関数 : YAO_NextMember
// 機能 : 次のメンバに移動
// 返値 : kYAO_Success			... 成功
//        kYAO_Failure			... 失敗
//        kYAO_FileIO_Err		... ファイル入出力エラー
//        kYAO_MemberNotFound	... 次のメンバが見つからなかった
//        kYAO_NoCurrent		... カレントが無い
//------------------------------------------------
// 20050831 mod (shiina, yatou) カレントをその次のメンバに移動するように変更
//                              内部関数FindNextMemberを使用するように変更
YAO_STATUS YAO_NextMember( YAO_FILE * ioFile );

//------------------------------------------------
// 関数 : YAO_PreviousMember
// 機能 : 前のメンバに移動
// 返値 : kYAO_Success			... 成功
//        kYAO_Failure			... 失敗
//        kYAO_FileIO_Err		... ファイル入出力エラー
//        kYAO_MemberNotFound	... 前のメンバが見つからなかった
//        kYAO_NoCurrent		... カレントが無い
//------------------------------------------------
// 20050831 mod (shiina, yatou) カレントをその前のメンバに移動するように変更
//                              内部関数FindPreviousMemberを使用するように変更
YAO_STATUS YAO_PreviousMember( YAO_FILE * ioFile );

//------------------------------------------------
// 関数 : YAO_FirstTag
// 機能 : 最初のタグオブジェクトに移動
// 返値 : kYAO_Success		... 成功
//        kYAO_Failure		... 失敗
//        kYAO_FileIO_Err	... ファイル入出力エラー
//        kYAO_TagNotFound	... 1つもタグが無い
//------------------------------------------------
// 20050831 add (shiina, yatou)
YAO_STATUS YAO_FirstTag( YAO_FILE * ioFile );

//------------------------------------------------
// 関数 : YAO_NextTag
// 機能 : 次のタグオブジェクトに移動
// 返値 : kYAO_Success		... 成功
//        kYAO_Failure		... 失敗
//        kYAO_FileIO_Err	... ファイル入出力エラー
//        kYAO_NoCurrent	... カレントが無い
//        kYAO_TagNotFound	... 次のタグが無い
//------------------------------------------------
// 20050831 add (shiina, yatou)
YAO_STATUS YAO_NextTag( YAO_FILE * ioFile );

//------------------------------------------------
// 関数 : YAO_PreviousTag
// 機能 : 前のタグオブジェクトに移動
// 返値 : kYAO_Success		... 成功
//        kYAO_Failure		... 失敗
//        kYAO_FileIO_Err	... ファイル入出力エラー
//        kYAO_NoCurrent	... カレントが無い
//        kYAO_TagNotFound	... 前のタグが無い
//------------------------------------------------
// 20050831 add (shiina, yatou)
YAO_STATUS YAO_PreviousTag( YAO_FILE * ioFile );

#endif
